@@ -1967,8 +1967,12 @@ public:
 			QByteArray data;
 			if(zresp.type == ZhttpResponsePacket::Close)
 			{
-				data.resize(2);
+				data.resize(2 + zresp.body.size());
 				writeBigEndian(data.data(), zresp.code != -1 ? zresp.code : 1000, 2);
+				if(!zresp.body.isEmpty())
+				{
+					memcpy(data.data() + 2, zresp.body.data(), zresp.body.size());
+				}
 			}
 
 			QByteArray frame = makeWsHeader(true, opcode, data.size()) + data;
@@ -2669,6 +2673,12 @@ private slots:
 					return;
 				}
 
+				if(s->downClosed)
+				{
+					log_debug("m2: %s id=%s ignoring frame after close", m2_send_idents[s->conn->identIndex].data(), mreq.id.data());
+					return;
+				}
+
 				ZhttpRequestPacket zreq;
 
 				if(opcode == 1 || opcode == 2)
@@ -2681,11 +2691,12 @@ private slots:
 				else if(opcode == 8)
 				{
 					zreq.type = ZhttpRequestPacket::Close;
-					if(mreq.body.size() == 2)
+					if(mreq.body.size() >= 2)
 					{
-						int hi = (unsigned char)zreq.body[0];
-						int lo = (unsigned char)zreq.body[1];
+						int hi = (unsigned char)mreq.body[0];
+						int lo = (unsigned char)mreq.body[1];
 						zreq.code = (hi << 8) + lo;
+						zreq.body = mreq.body.mid(2);
 					}
 
 					s->downClosed = true;
