@@ -1,8 +1,10 @@
 /*
- * Copyright (C) 2012-2016 Fanout, Inc.
+ * Copyright (C) 2016 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,11 +17,16 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
-#include <QtGlobal>
 #include <QCoreApplication>
-#include <QDateTime>
 #include <QTimer>
 #include "app.h"
 
@@ -34,28 +41,22 @@ public slots:
 	void start()
 	{
 		app = new App(this);
-		connect(app, SIGNAL(quit()), SLOT(app_quit()));
+		connect(app, &App::quit, this, &AppMain::app_quit);
 		app->start();
 	}
 
-	void app_quit()
+	void app_quit(int returnCode)
 	{
 		delete app;
-		emit quit();
+		QCoreApplication::exit(returnCode);
 	}
-
-signals:
-	void quit();
 };
 
 int main(int argc, char **argv)
 {
 	QCoreApplication qapp(argc, argv);
 
-	qsrand(QDateTime::currentMSecsSinceEpoch() & 0xffffffff);
-
 	AppMain appMain;
-	QObject::connect(&appMain, SIGNAL(quit()), &qapp, SLOT(quit()));
 	QTimer::singleShot(0, &appMain, SLOT(start()));
 	return qapp.exec();
 }
