@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2018 Fanout, Inc.
+ * Copyright (C) 2014-2019 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
@@ -41,6 +41,7 @@
 #include "uuidutil.h"
 
 #define BUFFER_SIZE 200000
+#define FRAME_SIZE_MAX 16384
 #define RESPONSE_BODY_MAX 1000000
 #define REJECT_BODY_MAX 100000
 #define RETRY_TIMEOUT 1000
@@ -202,9 +203,11 @@ public:
 	QList<Frame> inFrames;
 	QList<Frame> outFrames;
 	int closeCode;
+	QString closeReason;
 	bool closeSent;
 	bool peerClosing;
 	int peerCloseCode;
+	QString peerCloseReason;
 	bool disconnecting;
 	bool disconnectSent;
 	bool updateQueued;
@@ -282,6 +285,7 @@ public:
 		requestData.headers.removeAll("Upgrade");
 		requestData.headers.removeAll("Accept");
 		requestData.headers.removeAll("Connection-Id");
+		requestData.headers.removeAll("Content-Length");
 
 		// don't forward headers starting with Meta-*
 		for(int n = 0; n < requestData.headers.count(); ++n)
@@ -312,7 +316,7 @@ public:
 
 	void writeFrame(const Frame &frame)
 	{
-		assert(state != Closing);
+		assert(state == Connected);
 
 		outFrames += frame;
 
@@ -325,19 +329,23 @@ public:
 		return inFrames.takeFirst();
 	}
 
-	void close(int code)
+	void close(int code, const QString &reason)
 	{
 		assert(state != Closing);
 
 		state = Closing;
 		closeCode = code;
+		closeReason = reason;
 
 		update();
 	}
 
 	int writeBytesAvailable() const
 	{
-		int avail = BUFFER_SIZE;
+		if(reqContentSize >= BUFFER_SIZE)
+			return 0;
+
+		int avail = BUFFER_SIZE - reqContentSize;
 		foreach(const Frame &f, outFrames)
 		{
 			if(f.data.size() >= avail)
@@ -382,6 +390,29 @@ private:
 		return true;
 	}
 
+	void appendInMessage(Frame::Type type, const QByteArray &message)
+	{
+		// split into frames to avoid credits issue
+		QList<Frame> frames;
+
+		for(int n = 0; frames.isEmpty() || n < message.size(); n += FRAME_SIZE_MAX)
+		{
+			Frame::Type ftype;
+			if(n == 0)
+				ftype = type;
+			else
+				ftype = Frame::Continuation;
+
+			QByteArray data = message.mid(n, FRAME_SIZE_MAX);
+			bool more = (n + FRAME_SIZE_MAX < message.size());
+
+			frames += Frame(ftype, data, more);
+		}
+
+		foreach(const Frame &f, frames)
+			inFrames += f;
+	}
+
 	bool canSendCompleteMessage() const
 	{
 		foreach(const Frame &f, outFrames)
@@ -535,9 +566,12 @@ private:
 
 				if(closeCode != -1)
 				{
-					QByteArray buf(2, 0);
+					QByteArray rawReason = closeReason.toUtf8();
+
+					QByteArray buf(2 + rawReason.size(), 0);
 					buf[0] = (closeCode >> 8) & 0xff;
 					buf[1] = closeCode & 0xff;
+					memcpy(buf.data() + 2, rawReason.data(), rawReason.size());
 					events += WsEvent("CLOSE", buf);
 				}
 				else
@@ -746,29 +780,34 @@ private slots:
 			}
 			else if(e.type == "TEXT")
 			{
-				inFrames += Frame(Frame::Text, e.content, false);
+				appendInMessage(Frame::Text, e.content);
 				emitReadyRead = true;
 			}
 			else if(e.type == "BINARY")
 			{
-				inFrames += Frame(Frame::Binary, e.content, false);
+				appendInMessage(Frame::Binary, e.content);
 				emitReadyRead = true;
 			}
 			else if(e.type == "PING")
 			{
-				inFrames += Frame(Frame::Ping, e.content, false);
+				appendInMessage(Frame::Ping, e.content);
 				emitReadyRead = true;
 			}
 			else if(e.type == "PONG")
 			{
-				inFrames += Frame(Frame::Pong, e.content, false);
+				appendInMessage(Frame::Pong, e.content);
 				emitReadyRead = true;
 			}
 			else if(e.type == "CLOSE")
 			{
 				peerClosing = true;
-				if(e.content.size() == 2)
-					peerCloseCode = ((quint16)e.content[0] << 8) + (quint16)e.content[1];
+				if(e.content.size() >= 2)
+				{
+					int hi = (unsigned char)e.content[0];
+					int lo = (unsigned char)e.content[1];
+					peerCloseCode = (hi << 8) + lo;
+					peerCloseReason = QString::fromUtf8(e.content.mid(2));
+				}
 
 				closed = true;
 				break;
@@ -1087,7 +1126,7 @@ int WebSocketOverHttp::framesAvailable() const
 
 bool WebSocketOverHttp::canWrite() const
 {
-	return (writeBytesAvailable() > 0);
+	return (d->state == Connected && writeBytesAvailable() > 0);
 }
 
 int WebSocketOverHttp::writeBytesAvailable() const
@@ -1100,6 +1139,11 @@ int WebSocketOverHttp::peerCloseCode() const
 	return d->peerCloseCode;
 }
 
+QString WebSocketOverHttp::peerCloseReason() const
+{
+	return d->peerCloseReason;
+}
+
 WebSocket::ErrorCondition WebSocketOverHttp::errorCondition() const
 {
 	return d->errorCondition;
@@ -1115,9 +1159,9 @@ WebSocket::Frame WebSocketOverHttp::readFrame()
 	return d->readFrame();
 }
 
-void WebSocketOverHttp::close(int code)
+void WebSocketOverHttp::close(int code, const QString &reason)
 {
-	d->close(code);
+	d->close(code, reason);
 }
 
 void WebSocketOverHttp::setHeaders(const HttpHeaders &headers)
