@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2018 Fanout, Inc.
+ * Copyright (C) 2014-2019 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
@@ -425,7 +425,8 @@ public:
 		foreach(const HttpHeader &h, route.headers)
 		{
 			requestData.headers.removeAll(h.first);
-			requestData.headers += HttpHeader(h.first, h.second);
+			if(!h.second.isEmpty())
+				requestData.headers += HttpHeader(h.first, h.second);
 		}
 
 		clientAddress = inSock->peerAddress();
@@ -783,7 +784,7 @@ private slots:
 				}
 				else if(outSock->state() == WebSocket::Connected)
 				{
-					outSock->close(inSock->peerCloseCode());
+					outSock->close(inSock->peerCloseCode(), inSock->peerCloseReason());
 				}
 			}
 		}
@@ -792,10 +793,11 @@ private slots:
 	void in_closed()
 	{
 		int code = inSock->peerCloseCode();
+		QString reason = inSock->peerCloseReason();
 		cleanupInSock();
 
 		if(!detached && outSock && outSock->state() != WebSocket::Closing)
-			outSock->close(code);
+			outSock->close(code, reason);
 
 		tryFinish();
 	}
@@ -892,17 +894,18 @@ private slots:
 	void out_peerClosed()
 	{
 		if(!detached && inSock && inSock->state() != WebSocket::Closing)
-			inSock->close(outSock->peerCloseCode());
+			inSock->close(outSock->peerCloseCode(), outSock->peerCloseReason());
 	}
 
 	void out_closed()
 	{
 		int code = outSock->peerCloseCode();
+		QString reason = outSock->peerCloseReason();
 		delete outSock;
 		outSock = 0;
 
 		if(!detached && inSock && inSock->state() != WebSocket::Closing)
-			inSock->close(code);
+			inSock->close(code, reason);
 
 		tryFinish();
 	}
@@ -992,25 +995,18 @@ private slots:
 		// split into frames to avoid credits issue
 		QList<WebSocket::Frame> frames;
 
-		if(message.size() > 0)
+		for(int n = 0; frames.isEmpty() || n < message.size(); n += FRAME_SIZE_MAX)
 		{
-			for(int n = 0; n < message.size(); n += FRAME_SIZE_MAX)
-			{
-				WebSocket::Frame::Type ftype;
-				if(n == 0)
-					ftype = type;
-				else
-					ftype = WebSocket::Frame::Continuation;
+			WebSocket::Frame::Type ftype;
+			if(n == 0)
+				ftype = type;
+			else
+				ftype = WebSocket::Frame::Continuation;
 
-				QByteArray data = message.mid(n, FRAME_SIZE_MAX);
-				bool more = (n + FRAME_SIZE_MAX < message.size());
+			QByteArray data = message.mid(n, FRAME_SIZE_MAX);
+			bool more = (n + FRAME_SIZE_MAX < message.size());
 
-				frames += WebSocket::Frame(ftype, data, more);
-			}
-		}
-		else
-		{
-			frames += WebSocket::Frame(type, QByteArray(), false);
+			frames += WebSocket::Frame(ftype, data, more);
 		}
 
 		for(int n = 0; n < frames.count(); ++n)
